using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using Infragistics.Win.UltraWinTree;

namespace Infragistics.Win.Samples
{
	#region WinListViewDragSource
	/// <summary>
	/// Class which implements the IDragSource interface for the WinListView control.
	/// </summary>
	public class WinTreeDragSource : DragDropManager.IDragSource
	{
		#region Member variables
		
		private UltraTree _tree = null;
		private Cursor _dragMoveCursor = null;
		private Cursor _dragCopyCursor = null;
		
		#endregion Member variables
	
		#region Constructors

		/// <summary>
		/// Creates a new instance of the <see cref="WinListViewDragSource"/> class.
		/// </summary>
		/// <param name="tree">The UltraTree control with which this instance is associated.</param>
		public WinTreeDragSource( UltraTree tree ) : this( tree, null, null ){}

		/// <summary>
		/// Creates a new instance of the <see cref="WinListViewDragSource"/> class.
		/// </summary>
		/// <param name="tree">The UltraTree control with which this instance is associated.</param>
		/// <param name="dragMoveCursor">The cursor to display during a drag move operation.</param>
		/// <param name="dragCopyCursor">The cursor to display during a drag copy operation.</param>
		public WinTreeDragSource( UltraTree tree, Cursor dragMoveCursor, Cursor dragCopyCursor )
		{
			if ( tree == null )
				throw new ArgumentNullException( "tree  " );

			this._tree = tree;
			this._dragMoveCursor = dragMoveCursor;
			this._dragCopyCursor = dragCopyCursor;
		}

		#endregion Constructors

		#region IDragSource interface implementation

		//	Implements the Control property.
		UltraControlBase DragDropManager.IDragSource.Control
		{
			get{ return this._tree; }
		}

		//	Implements the DragMoveCursor property.
		Cursor DragDropManager.IDragSource.DragMoveCursor
		{
			get{ return this._dragMoveCursor; }
		}

		//	Implements the DragCopyCursor property.
		Cursor DragDropManager.IDragSource.DragCopyCursor
		{
			get{ return this._dragCopyCursor; }
		}

		//	Implements the HitTestElementType property.
		Type DragDropManager.IDragSource.HitTestElementType
		{
			get{ return typeof(TreeNodeUIElement); }
		}

		//	Implements the OnDragOperationStarting method.
		void DragDropManager.IDragSource.OnDragOperationStarting( DragDropManager.DragOperationStartingInfo info )
		{
			UltraTree tree = this._tree;

			//	Get a snapshot of the SelectedNodes collection into an array
			SelectedNodesCollection selectedNodes = tree.SelectedNodes;
			UltraTreeNode[] dragData = new UltraTreeNode[ selectedNodes.Count ];
			selectedNodes.CopyTo( dragData, 0 );

			//	Assign the UltraTreeItem array to the Drag property of the event arguments;
			//	this array will be passed around for the lifetime of the drag operation
			info.Data = dragData;
			
			//	Get another array, this one containing the UIElement for each of the
			//	selected items; these elements are used to generate an image which is
			//	displayed during the drag operation, so the end user can see what is
			//	being dragged. Assign this array to the 'DragImageElements' property
			//	of the event arguments.
			info.DragImageElements = WinTreeDragSource.GetDragImageElements( dragData );			
		}

		//	Returns an array of UIElements which should be displayed in the drag image
		//	based on the specified view and UltraTreeItem array.
		static private UIElement[] GetDragImageElements( UltraTreeNode[] nodes )
		{
			if ( nodes == null || nodes.Length == 0 )
				return null;

			ArrayList elementList = new ArrayList( nodes.Length );

			for ( int i = 0; i < nodes.Length; i ++ )
			{
				TreeNodeUIElement nodeElement = nodes[i].UIElement;
				if ( nodeElement == null )
					continue;

			    elementList.Add( nodeElement.GetDescendant( typeof(NodeSelectableAreaUIElement) ) );
            }

			return elementList.ToArray( typeof(UIElement) ) as UIElement[];
		}

		//	Implements the OnDragOperationContinuing method.
		void DragDropManager.IDragSource.OnDragOperationContinuing( DragDropManager.DragOperationContinuingInfo info )
		{
			//	Note that this method can be handled to cancel the drag operation,
			//	or display non-default cursors. The DragDropManager's default
			//	implementation handles the display of the move and copy cursors,
			//	which is all we need for this demonstration, so we don't need to
			//	do anything else here.
		}

		//	Handles the DragOperationCompleting event
		void DragDropManager.IDragSource.OnDragOperationCompleting( DragDropManager.DragOperationCompletingInfo info )
		{
			UltraTree tree = this._tree;

			//	If the drag operation is being committed, and it is a move
			//	operation, remove the items from the drag source's Items collection.
			if ( info.Committed && info.Effect == DragDropEffects.Move )
			{
				UltraTreeNode[] nodes = info.Data as UltraTreeNode[];
				if ( nodes != null )
				{
					for ( int i = nodes.Length - 1; i >= 0; i -- )
					{
						tree.Nodes.Remove( nodes[i] );
					}
				}
			}
		}

		#endregion IDragSource interface implementation
	}
	#endregion WinListViewDragSource

}